@@ -42,11 +42,13 @@ LIGATURES = {
 
 _ligpat = re.compile(u'|'.join(LIGATURES))
 
+
 def sanitize_head(match):
     x = match.group(1)
     x = _span_pat.sub('', x)
     return '<head>\n%s\n</head>' % x
 
+
 def chap_head(match):
     chap = match.group('chap')
     title = match.group('title')
@@ -55,6 +57,7 @@ def chap_head(match):
     else:
         return '<h1>'+chap+'</h1>\n<h3>'+title+'</h3>\n'
 
+
 def wrap_lines(match):
     ital = match.group('ital')
     if not ital:
@@ -62,7 +65,8 @@ def wrap_lines(match):
     else:
         return ital+' '
 
-def smarten_punctuation(html, log):
+
+def smarten_punctuation(html, log=None):
     from calibre.utils.smartypants import smartyPants
     from calibre.ebooks.chardet import substitute_entites
     from calibre.ebooks.conversion.utils import HeuristicProcessor
@@ -76,12 +80,9 @@ def smarten_punctuation(html, log):
     html = smartyPants(html)
     html = html.replace(start, '<!--')
     html = html.replace(stop, '-->')
-    # convert ellipsis to entities to prevent wrapping
-    html = re.sub(r'(?u)(?<=\w)\s?(\.\s?){2}\.', '&hellip;', html)
-    # convert double dashes to em-dash
-    html = re.sub(r'\s--\s', u'\u2014', html)
     return substitute_entites(html)
 
+
 class DocAnalysis(object):
     '''
     Provides various text analysis functions to determine how the document is structured.
@@ -186,6 +187,7 @@ class DocAnalysis(object):
             # print str(maxValue)+" of the lines were in one bucket"
             return True
 
+
 class Dehyphenator(object):
     '''
     Analyzes words to determine whether hyphens should be retained/removed.  Uses the document
@@ -280,6 +282,7 @@ class Dehyphenator(object):
         html = intextmatch.sub(self.dehyphenate, html)
         return html
 
+
 class CSSPreProcessor(object):
 
     # Remove some of the broken CSS Microsoft products
@@ -324,6 +327,7 @@ class CSSPreProcessor(object):
 
         return u'\n'.join(ans)
 
+
 class HTMLPreProcessor(object):
 
     PREPROCESS = [
@@ -460,8 +464,6 @@ class HTMLPreProcessor(object):
                   # Center separator lines
                   (re.compile(u'<br>\s*(?P<break>([*#•✦=] *){3,})\s*<br>'), lambda match: '<p>\n<p style="text-align:center">' + match.group('break') + '</p>'),
 
-                  # Remove page links
-                  (re.compile(r'<a name=\d+></a>', re.IGNORECASE), lambda match: ''),
                   # Remove <hr> tags
                   (re.compile(r'<hr.*?>', re.IGNORECASE), lambda match: ''),
 
@@ -495,6 +497,7 @@ class HTMLPreProcessor(object):
                      (re.compile('<span[^><]*?id=subtitle[^><]*?>(.*?)</span>', re.IGNORECASE|re.DOTALL),
                       lambda match : '<h3 class="subtitle">%s</h3>'%(match.group(1),)),
                      ]
+
     def __init__(self, log=None, extra_opts=None, regex_wizard_callback=None):
         self.log = log
         self.extra_opts = extra_opts
@@ -536,6 +539,7 @@ class HTMLPreProcessor(object):
 
         user_sr_rules = {}
         # Function for processing search and replace
+
         def do_search_replace(search_pattern, replace_txt):
             try:
                 search_re = re.compile(search_pattern)
