@@ -12,7 +12,7 @@ from threading import Thread
 from Queue import Queue, Empty
 
 
-from calibre import as_unicode
+from calibre import as_unicode, browser, random_user_agent
 from calibre.ebooks.metadata import check_isbn
 from calibre.ebooks.metadata.sources.base import (Source, Option, fixcase,
         fixauthors)
@@ -591,6 +591,19 @@ class Worker(Thread):  # Get details {{{
                         series = self.tostring(a[0], encoding=unicode, method='text', with_tail=False).strip()
                         if series:
                             ans = (series, series_index)
+        # This is found on newer Kindle edition pages on amazon.com
+        if ans == (None, None):
+            for b in root.xpath('//div[@id="reviewFeatureGroup"]/span/b'):
+                text = (b.text or '').strip()
+                m = re.match('Book\s+([0-9.]+)', text)
+                if m is not None:
+                    series_index = float(m.group(1))
+                    a = b.getparent().xpath('./a[@href]')
+                    if a:
+                        series = self.tostring(a[0], encoding=unicode, method='text', with_tail=False).partition('(')[0].strip()
+                        if series:
+                            ans = series, series_index
+
         if ans == (None, None):
             desc = root.xpath('//div[@id="ps-content"]/div[@class="buying"]')
             if desc:
@@ -787,6 +800,7 @@ class Amazon(Source):
             'br': _('Brazil'),
             'nl': _('Netherlands'),
             'cn': _('China'),
+            'ca': _('Canada'),
     }
 
     options = (
@@ -816,9 +830,15 @@ class Amazon(Source):
                 return key
 
     @property
-    def user_agent(self):
-        # IE 11 - windows 7
-        return 'Mozilla/5.0 (Windows NT 6.1; Trident/7.0; rv:11.0) like Gecko'
+    def browser(self):
+        if self._browser is None:
+            self._browser = br = browser(user_agent=random_user_agent(allow_ie=False))
+            br.set_handle_gzip(True)
+            # br.addheaders += [
+            #     ('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8'),
+            #     ('Upgrade-Insecure-Requests', '1'),
+            # ]
+        return self._browser.clone_browser()
 
     def save_settings(self, *args, **kwargs):
         Source.save_settings(self, *args, **kwargs)
@@ -1395,6 +1415,23 @@ if __name__ == '__main__':  # tests {{{
             ),
     ]  # }}}
 
+    ca_tests = [  # {{{
+            (   # Paperback with series
+                {'identifiers':{'isbn':'9781623808747'}},
+                [title_test('Parting Shot', exact=True), authors_test(['Mary Calmes'])]
+            ),
+            (  # # in title
+                {'title':'Expert C# 2008 Business Objects',
+                    'authors':['Lhotka']},
+                [title_test('Expert C# 2008 Business Objects'), authors_test(['Rockford Lhotka'])]
+            ),
+            (  # noscript description
+                {'identifiers':{'amazon_ca':'162380874X'}},
+                [title_test('Parting Shot', exact=True), authors_test(['Mary Calmes'])
+                ]
+            ),
+    ]  # }}}
+
     def do_test(domain, start=0, stop=None):
         tests = globals().get(domain+'_tests')
         if stop is None:
@@ -1404,7 +1441,6 @@ if __name__ == '__main__':  # tests {{{
                 p:(setattr(p, 'testing_domain', domain), setattr(p, 'touched_fields', p.touched_fields - {'tags'})))
 
     do_test('com')
-
     # do_test('de')
 
 # }}}
