@@ -20,11 +20,12 @@ from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.utils.config import prefs, XMLConfig
 from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndicator
 from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
-    IMAGE_EXTENSIONS, dnd_has_extension, DownloadDialog)
+    image_extensions, dnd_has_extension, DownloadDialog)
 from calibre.utils.localization import localize_user_manual_link
 
 history = XMLConfig('history')
 
+
 class ProgressIndicator(QWidget):  # {{{
 
     def __init__(self, *args):
@@ -58,6 +59,7 @@ class ProgressIndicator(QWidget):  # {{{
         self.setVisible(False)
 # }}}
 
+
 class FilenamePattern(QWidget, Ui_Form):  # {{{
 
     changed_signal = pyqtSignal()
@@ -170,6 +172,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
 
 # }}}
 
+
 class FormatList(QListWidget):  # {{{
     DROPABBLE_EXTENSIONS = BOOK_EXTENSIONS
     formats_dropped = pyqtSignal(object, object)
@@ -210,19 +213,21 @@ class FormatList(QListWidget):  # {{{
 
 # }}}
 
+
 class ImageDropMixin(object):  # {{{
     '''
     Adds support for dropping images onto widgets and a context menu for
     copy/pasting images.
     '''
-    DROPABBLE_EXTENSIONS = IMAGE_EXTENSIONS
+    DROPABBLE_EXTENSIONS = None
 
     def __init__(self):
         self.setAcceptDrops(True)
 
     def dragEnterEvent(self, event):
         md = event.mimeData()
-        if dnd_has_extension(md, self.DROPABBLE_EXTENSIONS) or \
+        exts = self.DROPABBLE_EXTENSIONS or image_extensions()
+        if dnd_has_extension(md, exts) or \
                 dnd_has_image(md):
             event.acceptProposedAction()
 
@@ -289,6 +294,7 @@ class ImageDropMixin(object):  # {{{
                     pixmap_to_data(pmap, format='PNG'))
 # }}}
 
+
 class ImageView(QWidget, ImageDropMixin):  # {{{
 
     BORDER_WIDTH = 1
@@ -343,9 +349,9 @@ class ImageView(QWidget, ImageDropMixin):  # {{{
         cw, ch = self.rect().width(), self.rect().height()
         scaled, nw, nh = fit_image(w, h, cw, ch)
         if scaled:
-            pmap = pmap.scaled(nw, nh, Qt.IgnoreAspectRatio,
+            pmap = pmap.scaled(int(nw*pmap.devicePixelRatio()), int(nh*pmap.devicePixelRatio()), Qt.IgnoreAspectRatio,
                     Qt.SmoothTransformation)
-        w, h = pmap.width(), pmap.height()
+        w, h = int(pmap.width()/pmap.devicePixelRatio()), int(pmap.height()/pmap.devicePixelRatio())
         x = int(abs(cw - w)/2.)
         y = int(abs(ch - h)/2.)
         target = QRect(x, y, w, h)
@@ -371,6 +377,7 @@ class ImageView(QWidget, ImageDropMixin):  # {{{
         p.end()
 # }}}
 
+
 class CoverView(QGraphicsView, ImageDropMixin):  # {{{
 
     cover_changed = pyqtSignal(object)
@@ -392,6 +399,8 @@ class CoverView(QGraphicsView, ImageDropMixin):  # {{{
 # }}}
 
 # BasicList {{{
+
+
 class BasicListItem(QListWidgetItem):
 
     def __init__(self, text, user_data=None):
@@ -403,6 +412,7 @@ class BasicListItem(QListWidgetItem):
             return self.text() == other.text()
         return False
 
+
 class BasicList(QListWidget):
 
     def add_item(self, text, user_data=None, replace=False):
@@ -426,6 +436,7 @@ class BasicList(QListWidget):
             yield self.item(i)
 # }}}
 
+
 class LineEditECM(object):  # {{{
 
     '''
@@ -474,6 +485,7 @@ class LineEditECM(object):  # {{{
 
 # }}}
 
+
 class EnLineEdit(LineEditECM, QLineEdit):  # {{{
 
     '''
@@ -491,6 +503,7 @@ class EnLineEdit(LineEditECM, QLineEdit):  # {{{
 
 # }}}
 
+
 class ItemsCompleter(QCompleter):  # {{{
 
     '''
@@ -518,6 +531,7 @@ class ItemsCompleter(QCompleter):  # {{{
 
 # }}}
 
+
 class CompleteLineEdit(EnLineEdit):  # {{{
 
     '''
@@ -577,6 +591,7 @@ class CompleteLineEdit(EnLineEdit):  # {{{
 
 # }}}
 
+
 class EnComboBox(QComboBox):  # {{{
 
     '''
@@ -603,6 +618,7 @@ class EnComboBox(QComboBox):  # {{{
 
 # }}}
 
+
 class CompleteComboBox(EnComboBox):  # {{{
 
     def __init__(self, *args):
@@ -620,6 +636,7 @@ class CompleteComboBox(EnComboBox):  # {{{
 
 # }}}
 
+
 class HistoryLineEdit(QComboBox):  # {{{
 
     lost_focus = pyqtSignal()
@@ -673,12 +690,14 @@ class HistoryLineEdit(QComboBox):  # {{{
 
 # }}}
 
+
 class ComboBoxWithHelp(QComboBox):  # {{{
     '''
     A combobox where item 0 is help text. CurrentText will return '' for item 0.
     Be sure to always fetch the text with currentText. Don't use the signals
     that pass a string, because they will not correct the text.
     '''
+
     def __init__(self, parent=None):
         QComboBox.__init__(self, parent)
         self.currentIndexChanged[int].connect(self.index_changed)
@@ -722,6 +741,7 @@ class ComboBoxWithHelp(QComboBox):  # {{{
 
 # }}}
 
+
 class EncodingComboBox(QComboBox):  # {{{
     '''
     A combobox that holds text encodings support
@@ -747,6 +767,7 @@ class EncodingComboBox(QComboBox):  # {{{
 
 # }}}
 
+
 class PythonHighlighter(QSyntaxHighlighter):  # {{{
 
     Rules = []
@@ -906,6 +927,8 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
 # }}}
 
 # Splitter {{{
+
+
 class SplitterHandle(QSplitterHandle):
 
     double_clicked = pyqtSignal(object)
@@ -928,6 +951,7 @@ class SplitterHandle(QSplitterHandle):
     def mouseDoubleClickEvent(self, ev):
         self.double_clicked.emit(self)
 
+
 class LayoutButton(QToolButton):
 
     def __init__(self, icon, text, splitter=None, parent=None, shortcut=None):
@@ -963,6 +987,7 @@ class LayoutButton(QToolButton):
         else:
             self.set_state_to_hide()
 
+
 class Splitter(QSplitter):
 
     state_changed = pyqtSignal(object)
